import React, { useState } from 'react';
import { Formik, Form, Field } from 'formik';
import Input from './Input';
import Select from './Select';
import Checkbox from './Checkbox';

interface DynamicFormProps {
  jsonData: { [key: string]: any };
}

const DynamicForm: React.FC<DynamicFormProps> = ({ jsonData }) => {
  const [isEditingJson, setIsEditingJson] = useState(false);
  const [formData, setFormData] = useState(jsonData);
  const [rawJson, setRawJson] = useState(JSON.stringify(jsonData, null, 2));

  const handleJsonEditToggle = () => {
    setIsEditingJson(!isEditingJson);
  };

  const handleJsonChange = (event: React.ChangeEvent<HTMLTextAreaElement>) => {
    setRawJson(event.target.value);
  };

  const handleJsonSubmit = () => {
    try {
      const updatedData = JSON.parse(rawJson);
      setFormData(updatedData); // Update form data with parsed JSON
      setIsEditingJson(false);
    } catch (error) {
      alert('Invalid JSON format. Please correct the JSON.');
    }
  };

  const renderFormField = (key: string, value: any) => {
    if (typeof value === 'boolean') {
      return (
        <div className="mb-10" key={key}>
          <Field name={key} type="checkbox">
            {({ field }: any) => (
              <Checkbox label={key.charAt(0).toUpperCase() + key.slice(1)} {...field} />
            )}
          </Field>
        </div>
      );
    } else if (typeof value === 'number') {
      return (
        <div className="mb-10" key={key}>
          <Field name={key} type="number">
            {({ field }: any) => (
              <Input label={key.charAt(0).toUpperCase() + key.slice(1)} {...field} />
            )}
          </Field>
        </div>
      );
    } else if (typeof value === 'string') {
      return (
        <div className="mb-10" key={key}>
          <Field name={key} type="text">
            {({ field }: any) => (
              <Input label={key.charAt(0).toUpperCase() + key.slice(1)} {...field} />
            )}
          </Field>
        </div>
      );
    }
    return null;
  };

  return (
    <div>
      <button type="button" className="btn btn-secondary mb-3" onClick={handleJsonEditToggle}>
        {isEditingJson ? 'Close JSON Editor' : 'Edit Raw JSON'}
      </button>

      {isEditingJson ? (
        <div>
          <textarea
            className="form-control"
            rows={10}
            value={rawJson}
            onChange={handleJsonChange}
          />
          <button type="button" className="btn btn-primary mt-2" onClick={handleJsonSubmit}>
            Update JSON
          </button>
        </div>
      ) : (
        <Formik
          initialValues={formData}
          enableReinitialize={true}
          onSubmit={(values) => {
            console.log('Form Submitted', values);
          }}
        >
          {() => (
            <Form className="form">
              {Object.keys(formData).map((key) => renderFormField(key, formData[key]))}
              <div className="d-flex justify-content-end">
                <button type="submit" className="btn btn-primary">
                  Submit
                </button>
              </div>
            </Form>
          )}
        </Formik>
      )}
    </div>
  );
};

export default DynamicForm;
